import { defineNuxtModule, createResolver, installModule, addComponent, addTemplate, addTypeTemplate } from '@nuxt/kit';
import { readdir } from 'fs/promises';
import { join, dirname, parse } from 'path';



// -- Unbuild CommonJS Shims --
import __cjs_url__ from 'url';
import __cjs_path__ from 'path';
import __cjs_mod__ from 'module';
const __filename = __cjs_url__.fileURLToPath(import.meta.url);
const __dirname = __cjs_path__.dirname(__filename);
const require = __cjs_mod__.createRequire(import.meta.url);
const module = defineNuxtModule({
  meta: {
    name: "nuxt-bootstrap-icons",
    configKey: "bootstrapIcons",
    compatibility: {
      nuxt: "^3.0.0"
    }
  },
  // Default configuration options of the Nuxt module
  defaults: {
    display: "inline",
    prefix: "bootstrap-icon",
    showList: false
  },
  async setup(options) {
    const { resolve } = createResolver(import.meta.url);
    const paths = {
      bootstrapIcons: join(
        dirname(require.resolve("bootstrap-icons/package.json")),
        "icons"
      ),
      components: resolve("./runtime/components"),
      typings: resolve("./runtime/_types.d.ts")
    };
    let iconList = (await readdir(paths.bootstrapIcons)).map(
      (entry) => parse(entry).name
    );
    const _prefix = options.prefix.includes("-") ? options.prefix.split("-").map((word) => word.at(0)?.toUpperCase() + word.slice(1)).join("") : options.prefix.at(0)?.toUpperCase() + options.prefix.slice(1);
    if (options.display === "component") {
      await installModule("nuxt-svgo-loader");
      const iconPaths = iconList.map(
        (icon) => join(paths.bootstrapIcons, icon) + ".svg"
      );
      for (let index = 0; index < iconPaths.length; index++) {
        const filename = iconList[index].split("-").map((word) => word.charAt(0).toUpperCase() + word.slice(1)).join("");
        iconList[index] = filename;
        await addComponent({
          filePath: iconPaths[index],
          name: _prefix + filename
        });
      }
    } else
      await addComponent({
        filePath: resolve(paths.components, "BootstrapIcon.vue"),
        name: _prefix
      });
    if (options.showList)
      addTemplate({
        filename: "nuxt-bootstrap-icons.json",
        getContents: () => JSON.stringify(iconList),
        write: true
      });
    addTypeTemplate({
      filename: "types/nuxt-bootstrap-icons.d.ts",
      src: resolve(paths.typings)
    });
  }
});

export { module as default };
